function WPrice = WeightedMethodDOBarrier(thet,S0,Strike,r,q,S,T,LVi,Ti,Ki,dt,GridType,BarType,Barrier,imethod,intmethod)

% Down and out barrier call option using Weighted Method and local volatility
% INPUTS
%   thet = Weighing parameter
%   S0 = Spot price at which to price the call
%   Strike = Strike price
%   r,q = risk free rate, dividend yield
%   S = grid for the stock price
%   T = grid for the maturity
%   LVi = interpolated Local Volatility surface
%   Ti  = maturities for LVi
%   Ki  = stock prices for LVi
%   dt   = time increment for Ti
%   BarType = Barrier type (DO)
%   Barrier = Barrier Level
%   imethod = 'Inverse' (slow) or 'Tridiagonal' (fast)
%   intmethod = 2d interpolation of local vo
% OUTPUT
%   WPrice = Down and out barrier option price

% Required vector lengths and time increment
NS = length(S);
NT = length(T);

% Initialize the u vector
u = zeros(NS*NS,1);

% U(0) vector - value of U(T) at maturity
Si = repmat(S',NS,1);
U = max(0, Si - Strike);

% Obtain the derivatives matrices
if strcmp(GridType,'Uniform')
    [derS derSS R] = BuildDerivativesUniformBarrier(S,T);
else
    [derS derSS R] = BuildDerivativesNonUniformBarrier(S,T);
end
% Identity matrix
I = eye(NS*NS);

% Loop through the time increments, updating U(t) to U(t+1) at each step
for t=2:NT
    u = U;
    T0 = T(t);
    % Interpolate the local volatility at T(t) and S
    volvec = interp2(Ti,Ki',LVi,T0,S,intmethod);
    % Construct a matrix of interpolated local vols
    volmat = repmat(volvec,1,NS);
    sigma = repmat(volmat,NS,NS);
    % The L operator matrix
    L = (r-q).*derS + (0.5.*sigma.*sigma).*derSS - r.*R;
    % Impose the barrier on the L matrix
    if strcmp(BarType,'DO')
        J = find(Si<=Barrier);
        L(J,:) = 0;
        L(:,J) = 0;
    end
    % Matrices for weighted method
    A = (I - thet.*dt.*L);
    B = (I + (1-thet).*dt.*L);
    % Explicit method
    if thet == 0;
        U = B*u;
    else
        % Slow method: find inv(A)
        if strcmp(imethod,'Inverse')
            if thet==1
                U = inv(A)*u;      % Implicit Method
            else
                U = A\B*u;         % Weighted Method
            end
        % Fast method: exploit tridiagonal form of A
        elseif strcmp(imethod,'Tridiagonal')
            U = MSolveTridiagonal(A,B*u);
        end
    end
end

% Retain only the first NS terms of the U vector
U1 = U(1:NS);

% Interpolate to get the final DO call price
WPrice = interp1(S',U1,S0);
